# Include the BeakerLib environment
. /usr/share/beakerlib/beakerlib.sh

# Set the full test name
TEST="testA"

# Package being tested
PACKAGE="samba"

SMBUSER="smbtstuser"

# Required packages.
PACKAGES=( "openldap"            \
           "openldap-clients"    \
           "openldap-servers"    \
           "samba"               \
           "samba-client"        \
           "smbldap-tools"       \
           "system-config-samba" \
           "rsyslog"             )

rlJournalStart

  rlPhaseStartSetup

    # Backup service. 
    rlServiceStop "slapd"
    rlServiceStop "rsyslog"
    rlServiceStop "smb"

    # Check & install reqiured packages.
    yum clean all
    for P in ${PACKAGES[@]}; do
        rlCheckRpm $P || rlRun "yum -y install $P" 0 "Installing $P"
    done

    # Backup configuration and database.
    rlFileBackup "/etc/hosts"
    rlFileBackup "/etc/rsyslog.conf"
    rlFileBackup "/etc/openldap/slapd.conf"
    rlFileBackup "/etc/openldap/ldap.conf"
    rlFileBackup "/etc/sysconfig/ldap"
    rlFileBackup "/etc/openldap/cacerts/" && rlRun "rm -rf /etc/openldap/cacerts/*" 0
    rlFileBackup "/etc/openldap/slapd.d/" && rlRun "rm -rf /etc/openldap/slapd.d/*" 0
    rlFileBackup "/var/run/openldap/" && rlRun "rm -rf /var/run/openldap/*" 0
    rlFileBackup "/var/lib/ldap/" && rlRun "rm -rf /var/lib/ldap/*" 0
    rlFileBackup "/etc/nsswitch.conf"

    if [ -f $HOME/ldaprc  ] ; then 
	        rlFileBackup "$HOME/ldaprc" && rlRun "rm -f $HOME/ldaprc" 0
    fi

    # Backup samba configuration
    rlFileBackup "/etc/openldap/schema/" && rlRun "rm -f /etc/openldap/schema/samba.schema" 0 
    rlFileBackup "/etc/samba/smb.conf"
    rlFileBackup "/etc/smbldap-tools/smbldap.conf"
    rlFileBackup "/etc/smbldap-tools/smbldap_bind.conf"

    # Enable logging.
    rlRun "echo 'local4.* /var/log/slapd.log' >> /etc/rsyslog.conf" 0
    rlRun "service rsyslog start && sleep 5"

    # Configure.
    rlRun "echo '127.0.0.1 my-domain.com' >> /etc/hosts" 0
    rlRun "cp -f nsswitch.conf /etc/nsswitch.conf" 0
    rlRun "cp -f ldaprc $HOME/ldaprc" 0
    rlRun "cp -f samba.schema /etc/openldap/schema/samba.schema" 0
    rlRun "cp -f slapd.conf /etc/openldap/slapd.conf" 0
    rlRun "cp -f ldap.conf /etc/openldap/ldap.conf" 0
    rlRun "cp -f ldap /etc/sysconfig/ldap" 0
    rlRun "cp -f *.pem /etc/openldap/cacerts/" 0
    rlRun "chown ldap:ldap /etc/openldap/cacerts/*" 0
    rlRun "chmod 400 /etc/openldap/cacerts/*" 0
    rlRun "slaptest -f /etc/openldap/slapd.conf -F /etc/openldap/slapd.d/" 0-255
    rlRun "chown -R ldap:ldap /etc/openldap/slapd.d/" 0
    rlRun "cacertdir_rehash /etc/openldap/cacerts/" 0

    # Load testing data.
    rlRun "slapadd -l data.ldif" 0
    rlRun "chown -R ldap:ldap /var/lib/ldap" 0

    # SELinux cleaning.
    rlRun "restorecon -Rv /etc/"

    # Start.
    rlRun "service slapd start && sleep 5" 0

  rlPhaseEnd

  rlPhaseStartTest  

    # Testing.

    # No SSL/TLS.
    rlRun "cp -f smb.conf /etc/samba/smb.conf" 0
    rlRun "smbpasswd -w x" 0 "LDAP admin password added to smb password db"

    # Start Samba
    rlRun "service smb start && sleep 5" 0

    # Setup smbldap-tools
    rlRun "cp -f smbldap.conf /etc/smbldap-tools/smbldap.conf" 0
    rlRun "cp -f smbldap_bind.conf /etc/smbldap-tools/smbldap_bind.conf" 0

    rlRun "chmod 0644 /etc/smbldap-tools/smbldap.conf" 0
    rlRun "chmod 0600 /etc/smbldap-tools/smbldap_bind.conf" 0

    rlRun "useradd -m $SMBUSER" 0 "Add $SMBUSER to system"
    rlRun "smbldap-useradd -s /bin/false $SMBUSER" 0 "Add $SMBUSER to ldap"
    rlRun "echo -e 'aaa\naaa' | smbpasswd -s -a $SMBUSER" 0
    rlRun "smbclient -L my-domain.com -U $SMBUSER%aaa" 0 "Test samba-user access to samba server with no TLS/SSL option"

    # TLS.
    rlRun "service smb stop && sleep 5" 0
    rlRun "cp -f smb_tls.conf /etc/samba/smb.conf" 0
    rlRun "service smb start && sleep 5" 0
    rlRun "smbclient -L my-domain.com -U $SMBUSER%aaa" 0 "Test samba-user access to samba server with TLS option"

    # SSL.
    rlRun "service smb stop && sleep 5" 0
    rlRun "cp -f smb_ssl.conf /etc/samba/smb.conf" 0
    rlRun "service smb start && sleep 5" 0
    rlRun "smbclient -L my-domain.com -U $SMBUSER%aaa" 0 "Test samba-user access to samba server with SSL option"

  rlPhaseEnd

  rlPhaseStartCleanup

    # Stop. 
    rlRun "service smb stop && sleep 5" 0
    rlRun "service slapd stop && sleep 5" 0 

    # Delete smbtstuser from system and delete it's home directory
    rlRun "userdel -r $SMBUSER" 0

    # Delete user ldap configuration file
    rlRun "rm -f $HOME/ldaprc" 0

    # Delete test certificates
    rlRun "rm -f /etc/openldap/cacerts/*.pem" 0
 
    # Restore configuration backup.
    rlFileRestore
   
    # Restore service backup.
    rlServiceRestore "slapd"
    rlServiceRestore "rsyslog"
    rlServiceRestore "smb"

  rlPhaseEnd

rlJournalEnd

# Print the test report
rlJournalPrintText
